@@ -11,10 +11,8 @@
 #include <KItinerary/ExtractorRepository>
 #include <KItinerary/ScriptExtractor>
 
-#include <QDebug>
 #include <QObject>
 #include <QTest>
-#include <QVariant>
 
 using namespace KItinerary;
 
