@@ -5,6 +5,7 @@
 */
 
 #include "config-kitinerary.h"
+#include "testhelpers.h"
 
 #include <KItinerary/ExtractorDocumentNode>
 #include <KItinerary/ExtractorDocumentNodeFactory>
@@ -16,7 +17,6 @@
 #include <QDebug>
 #include <QFile>
 #include <QJsonDocument>
-#include <QProcess>
 #include <QTest>
 
 using namespace KItinerary;
@@ -72,7 +72,8 @@ private Q_SLOTS:
         QFETCH(QString, refFile);
 
         QFile in(inputFile);
-        QVERIFY(in.open(QFile::ReadOnly));
+        const auto openFlags = inputFile.endsWith(QLatin1String(".txt")) ? QFile::Text : QFile::ReadOnly;
+        QVERIFY(in.open(QFile::ReadOnly | openFlags));
 
         ExtractorEngine engine;
         auto root = engine.documentNodeFactory()->createNode(in.readAll(), inputFile);
@@ -87,19 +88,7 @@ private Q_SLOTS:
         QFile ref(refFile);
         QVERIFY(ref.open(QFile::ReadOnly));
         const auto refResult = QJsonDocument::fromJson(ref.readAll()).array();
-
-        if (result != refResult) {
-            QFile failFile(refFile + QLatin1String(".fail"));
-            QVERIFY(failFile.open(QFile::WriteOnly));
-            failFile.write(QJsonDocument(result).toJson());
-            failFile.close();
-
-            QProcess proc;
-            proc.setProcessChannelMode(QProcess::ForwardedChannels);
-            proc.start(QStringLiteral("diff"), {QStringLiteral("-u"), refFile, failFile.fileName()});
-            QVERIFY(proc.waitForFinished());
-        }
-        QCOMPARE(result, refResult);
+        QVERIFY(Test::compareJson(refFile, result, refResult));
     }
 
     void testInfiniteLoop()
