@@ -120,14 +120,14 @@ function parseTicket(text, uic918ticket) {
     var returnResIndex = 0;
     while (true) {
         // find itinerary headers
-        var header = text.substr(pos).match(/Ihre Reiseverbindung[\S ]+(Hin|Rück)fahrt am [0-9]{2}.[0-9]{2}.([0-9]{4}).*\n/);
+        var header = text.substr(pos).match(/Ihre Reiseverbindung[\S ]+(Hin|Rück|Einfache )[fF]ahrt am [0-9]{2}.[0-9]{2}.([0-9]{4}).*\n/);
         if (!header)
             break;
         var idx = header.index + header[0].length;
         var year = header[2];
 
         // determine ticket type
-        var domesticHeader = text.substr(pos + idx).match(/  Reservierung\n/);
+        var domesticHeader = text.substr(pos + idx).match(/  Reservierung(?: \/ Hinweise)?\n/);
         var intlHeader = text.substr(pos + idx).match(/(Produkte\/Reservierung|Fahrt\/Reservierung).*\n/);
         if (domesticHeader) {
             idx += domesticHeader.index + domesticHeader[0].length;
@@ -140,9 +140,9 @@ function parseTicket(text, uic918ticket) {
         }
 
         // for outward journeys we have station ids from the UIC 918-3 code
-        if (uic918ticket && header[1] === "Hin") {
-            reservations[0].reservationFor.departureStation.identifier = uic918ticket.outboundDepartureStationId;
-            reservations[reservations.length - 1].reservationFor.arrivalStation.identifier = uic918ticket.outboundArrivalStationId;
+        if (uic918ticket && header[1] !== "Rück") {
+            reservations[0].reservationFor.departureStation.identifier = uic918ticket.outboundDepartureStation.identifier;
+            reservations[reservations.length - 1].reservationFor.arrivalStation.identifier = uic918ticket.outboundArrivalStation.identifier;
             returnResIndex = reservations.length;
         } else {
             // propagate station ids from outward to return journey
@@ -218,24 +218,24 @@ function parseReservation(pdf) {
 function applyUic9183ToReservation(res, uicCode)
 {
     res.reservationNumber = uicCode.pnr;
-    res.reservationFor.provider.identifier = "uic:" + uicCode.carrierId;
+    res.reservationFor.provider = JsonLd.toJson(uicCode.issuer);
     const bl = uicCode.block('0080BL');
-    let sb = bl.findSubBlock('009');
-    if (sb) {
-        const bc = sb.content.match(/\d+-\d+-(.*)/)[1];
-        switch (bc) {
-            case "49":
-                res.programMembershipUsed.programName = "BahnCard 25";
-                break;
-            case "19":
-            case "78":
-                res.programMembershipUsed.programName = "BahnCard 50";
-                break;
+    if (bl) {
+        const sb = bl.findSubBlock('009');
+        if (sb) {
+            const bc = sb.content.match(/\d+-\d+-(.*)/)[1];
+            switch (bc) {
+                case "49":
+                    res.programMembershipUsed.programName = "BahnCard 25";
+                    break;
+                case "19":
+                case "78":
+                    res.programMembershipUsed.programName = "BahnCard 50";
+                    break;
+            }
         }
     }
-    sb = bl.findSubBlock('001');
-    if (sb)
-        res.reservedTicket.name = sb.content;
+    res.reservedTicket.name = uicCode.name
     if (res.reservedTicket.ticketedSeat)
         res.reservedTicket.ticketedSeat.seatingType = uicCode.seatingType;
     res.underName = JsonLd.toJson(uicCode.person);
@@ -290,15 +290,13 @@ function parseUic9183(code, node) {
 
     // domestic ticket code
     const bl = code.block('0080BL');
-    if (bl) {
+    if (bl && code.outboundDepartureStation.name && code.outboundArrivalStation.name) {
         let res = JsonLd.newTrainReservation();
         res.reservedTicket = node.result[0];
         applyUic9183ToReservation(res, code);
         res.reservationFor.departureDay = JsonLd.toDateTime(bl.findSubBlock('031').content, 'dd.mm.yyyy', 'de');
-        res.reservationFor.departureStation.name = bl.findSubBlock('015').content;
-        res.reservationFor.departureStation.identifier = code.outboundDepartureStationId;
-        res.reservationFor.arrivalStation.name = bl.findSubBlock('016').content;
-        res.reservationFor.arrivalStation.identifier = code.outboundArrivalStationId;
+        res.reservationFor.departureStation = JsonLd.toJson(code.outboundDepartureStation);
+        res.reservationFor.arrivalStation = JsonLd.toJson(code.outboundArrivalStation);
 
         if (!bl.findSubBlock('017')) {
             return res;
@@ -308,13 +306,22 @@ function parseUic9183(code, node) {
         ret.reservedTicket = node.result[0];
         applyUic9183ToReservation(ret, code);
         ret.reservationFor.departureDay = JsonLd.toDateTime(bl.findSubBlock('032').content, 'dd.mm.yyyy', 'de');
-        ret.reservationFor.departureStation.name = bl.findSubBlock('017').content;
-        ret.reservationFor.departureStation.identifier = ret.reservationFor.departureStation.name === res.reservationFor.arrivalStation.name ? code.outboundArrivalStationId : undefined;
-        ret.reservationFor.arrivalStation.name = bl.findSubBlock('018').content;
-        ret.reservationFor.arrivalStation.identifier = ret.reservationFor.arrivalStation.name === res.reservationFor.departureStation.name ? code.outboundDepartureStationId : undefined;
-
+        ret.reservationFor.departureStation = JsonLd.toJson(code.returnDepartureStation);
+        ret.reservationFor.arrivalStation = JsonLd.toJson(code.returnArrivalStation);
         return [res, ret];
     }
+
+    const fcb = code.block('U_FLEX');
+    if (fcb && code.outboundDepartureStation.name && code.outboundArrivalStation.name) {
+        let res = JsonLd.newTrainReservation();
+        res.reservedTicket = node.result[0];
+        applyUic9183ToReservation(res, code);
+        // TODO needs different handling based on document type
+        res.reservationFor.departureDay = fcb.transportDocument[0].ticket.validFrom(fcb.issuingDetail.issueingDateTime);
+        res.reservationFor.departureStation = JsonLd.toJson(code.outboundDepartureStation);
+        res.reservationFor.arrivalStation = JsonLd.toJson(code.outboundArrivalStation);
+        return res;
+    }
 }
 
 function parseEvent(event) {
