@@ -9,7 +9,6 @@
 
 #include <QDateTime>
 #include <QDebug>
-#include <QSize>
 
 #include <cstring>
 
@@ -151,18 +150,20 @@ QString Uic9183TicketLayout::text(int row, int column, int width, int height) co
     }
 
     for (auto f = firstField(); !f.isNull(); f = f.next()) {
-        if (f.row() + f.height() - 1 < row || f.row() > row + height - 1) {
+        // there's non-compliant samples out there with zero field sizes...
+        const auto effectiveHeight = std::max(f.height(), 1);
+        if (f.row() + effectiveHeight - 1 < row || f.row() > row + height - 1) {
             continue;
         }
-        if (f.column() + f.width() - 1 < column || f.column() > column + width - 1) {
+        const auto effectiveFieldWidth = f.width() > 0 ? f.width() : f.size();
+        if (f.column() + effectiveFieldWidth - 1 < column || f.column() > column + width - 1) {
             continue;
         }
 
         // split field into lines
         // TODO this needs to follow the U_TLAY word-wrapping algorithm?
         const auto content = f.text();
-        const auto lines = content.splitRef(QLatin1Char('\n'));
-
+        const auto lines = QStringView(content).split(QLatin1Char('\n'));
         // cut out the right part of the line
         for (int i = 0; i < lines.size(); ++i) {
             if (f.row() + i < row) {
@@ -192,7 +193,25 @@ QSize Uic9183TicketLayout::size() const
     int height = 0;
     for (auto f = firstField(); !f.isNull(); f  = f.next()) {
         width = std::max(width, f.column() + f.width());
-        height = std::max(height, f.row() + f.height());
+        height = std::max(height, f.row() + std::max(1, f.height()));
     }
     return QSize(width, height);
 }
+
+std::vector<Uic9183TicketLayoutField> Uic9183TicketLayout::fields(int row, int column, int width, int height) const
+{
+    std::vector<Uic9183TicketLayoutField> result;
+    for (auto f = firstField(); !f.isNull(); f = f.next()) {
+        // there's non-compliant samples out there with zero field sizes...
+        const auto effectiveHeight = std::max(f.height(), 1);
+        if (f.row() + effectiveHeight - 1 < row || f.row() > row + height - 1) {
+            continue;
+        }
+        const auto effectiveFieldWidth = f.width() > 0 ? f.width() : f.size();
+        if (f.column() + effectiveFieldWidth - 1 < column || f.column() > column + width - 1) {
+            continue;
+        }
+        result.push_back(f);
+    }
+    return result;
+}
